<?php

namespace FiloBlu\CronExtension;

use Exception;
use FiloBlu\CronExtension\Shell\ProcessAwaiter;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Phrase;
use Magento\Framework\Shell\CommandRendererInterface;
use Magento\Framework\ShellInterface;
use Psr\Log\LoggerInterface;

/**
 * Shell command line wrapper encapsulates command execution and arguments escaping
 */
class Shell implements ShellInterface
{
    /**
     * Logger instance
     *
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var ProcessAwaiter
     */
    protected $awaiter;

    /**
     * @var CommandRendererInterface
     */
    private $commandRenderer;

    /**
     * @param CommandRendererInterface $commandRenderer
     * @param LoggerInterface $logger Logger instance to be used to log commands and their output
     */
    public function __construct(
        CommandRendererInterface $commandRenderer,
        ProcessAwaiter $awaiter,
        LoggerInterface $logger = null
    )
    {
        $this->logger = $logger;
        $this->commandRenderer = $commandRenderer;
        $this->awaiter = $awaiter;
    }

    /**
     * Execute a command through the command line, passing properly escaped arguments, and return its output
     *
     * @param string $command Command with optional argument markers '%s'
     * @param string[] $arguments Argument values to substitute markers with
     * @return string Output of an executed command
     * @throws LocalizedException If a command returns non-zero exit code
     */
    public function execute($command, array $arguments = [])
    {
        $command = $this->commandRenderer->render($command, $arguments);
        $this->log($command);

        $disabled = explode(',', str_replace(' ', ',', ini_get('disable_functions')));
        if (in_array('exec', $disabled, false)) {
            throw new LocalizedException(new Phrase('The exec function is disabled.'));
        }

        exec($command, $output, $exitCode);

        // Wait only for standalone process
        if (strpos($command, '--bootstrap=standaloneProcessStarted=1', 0) !== false) {
            $this->awaiter->add((int)$output[0]);
            unset($output[0]);
        }

        $output = implode(PHP_EOL, $output);
        $this->log($output);

        if ($exitCode) {
            $commandError = new Exception($output, $exitCode);
            throw new LocalizedException(
                new Phrase("Command returned non-zero exit code:\n`%1`", [$command]),
                $commandError
            );
        }
        return $output;
    }

    /**
     * Log a message, if a logger is specified
     *
     * @param string $message
     * @return void
     */
    protected function log($message)
    {
        if ($this->logger) {
            $this->logger->info($message);
        }
    }
}
